// ignore_for_file: avoid_print, unused_local_variable

import 'package:app_telegram_template/widget/app_telegram_template.dart';
import 'package:flutter/material.dart';
import 'package:flutter_hicons/flutter_hicons.dart';
import 'package:general_lib/general_lib.dart';

import 'package:general_lib_flutter/general_lib_flutter.dart';
import 'package:shimmer/shimmer.dart';
import 'package:telegram_client/telegram_client/telegram_client_data.dart';

class SettingsSettingDevicePageAppTelegramTemplate
    extends AppTelegramTemplateStatefulWidget {
  const SettingsSettingDevicePageAppTelegramTemplate({
    super.key,
    required super.appTelegramTemplateData,
  });

  @override
  State<SettingsSettingDevicePageAppTelegramTemplate> createState() =>
      _SettingsSettingDevicePageAppTelegramTemplateState();
}

class _SettingsSettingDevicePageAppTelegramTemplateState
    extends State<SettingsSettingDevicePageAppTelegramTemplate> {
  final GlobalKey bottomBarKey = GlobalKey();

  final GlobalKey globalKey = GlobalKey();

  final scaffoldKey = GlobalKey<ScaffoldState>();

  PageController pageController = PageController();

  Size getRedBoxSize(BuildContext context) {
    final box = context.findRenderObject() as RenderBox;
    return box.size;
  }

  @override
  void initState() {
    // TODO: implement initState
    super.initState();

    WidgetsBinding.instance.addPostFrameCallback((_) {
      setState(() {
        // redboxSize = getRedBoxSize(globalKey.currentContext!);
      });
    });
  }

  @override
  Widget build(BuildContext context) {
    // print(widget.getRedBoxSize()!.height);
    return Scaffold(
      extendBodyBehindAppBar: true,
      extendBody: true,
      appBar: () {
        return PreferredSize(
          preferredSize: Size(
            context.width,
            450,
          ), // here the desired height
          child: Container(
            key: globalKey,
            decoration: BoxDecoration(
              color: context.theme.dialogBackgroundColor.withOpacity(0.85),
              boxShadow: [
                BoxShadow(
                  color: context.theme.shadowColor.withAlpha(110),
                  spreadRadius: 1,
                  blurRadius: 7,
                  offset: const Offset(0, 3), // changes position of shadow
                ),
              ],
            ),
            child: Column(
              mainAxisSize: MainAxisSize.min,
              crossAxisAlignment: CrossAxisAlignment.start,
              children: [
                SizedBox(
                  height: context.mediaQueryData.padding.top,
                ),
                MaterialButton(
                  shape: RoundedRectangleBorder(
                    borderRadius: BorderRadius.circular(25.0),
                  ),
                  hoverColor: Colors.transparent,
                  splashColor: Colors.transparent,
                  highlightColor: Colors.transparent,
                  onPressed: () async {},
                  padding: const EdgeInsets.all(15),
                  child: Row(
                    mainAxisSize: MainAxisSize.max,
                    mainAxisAlignment: MainAxisAlignment.start,
                    children: [
                      MaterialButton(
                        shape: RoundedRectangleBorder(
                          borderRadius: BorderRadius.circular(25.0),
                        ),
                        minWidth: 0,
                        onPressed: () async {
                          context.navigator().pop();
                        },
                        child: const Icon(Icons.arrow_back_ios_sharp),
                      ),
                      const Padding(
                        padding: EdgeInsets.all(10),
                        child: Text(
                          "Devices",
                          style: TextStyle(
                            fontSize: 16,
                            fontWeight: FontWeight.w600,
                          ),
                        ),
                      ),
                    ],
                  ),
                ),
              ],
            ),
          ),
        );
      }(),
      body: body(),
      // floatingActionButton: FloatingActionButton(
      //   onPressed: () async {},
      // ),
    );
  }

  Widget body() {
    return SingleChildScrollView(
      physics: const AlwaysScrollableScrollPhysics(
        parent: BouncingScrollPhysics(),
      ),
      child: ConstrainedBox(
        constraints: BoxConstraints(
          minHeight: context.height,
          minWidth: context.width,
        ),
        child: FutureBuilder(
          future: Future(() async {
            return await widget.appTelegramTemplateData.invoke(
              context: context,
              parameters: {
                "@type": "getActiveSessions",
              },
              isInvokeThrowOnError: false,
            );
          }),
          builder: (context, snapshot) {
            Map data = snapshot.data ?? {};
            data.printPretty();
            if (data["sessions"] is List == false) {
              data["sessions"] = [];
            }
            List<Map> sessions =
                (data["sessions"] as List).whereType<Map>().toList();

            ListView child = ListView.builder(
              physics: const NeverScrollableScrollPhysics(),
              shrinkWrap: true,
              itemCount: sessions.length,
              itemBuilder: (context, index) {
                Map session = sessions[index];
                // sessions[index].printPretty();
                ListTile listTile = ListTile(
                  onTap: () {
                    session.printPretty(2);
                  },
                  leading: const FlutterLogo(),
                  title: Text(
                    "${session["device_model"] ?? ""}",
                    style: const TextStyle(
                      overflow: TextOverflow.fade,
                    ),
                  ),
                  subtitle: Column(
                    mainAxisSize: MainAxisSize.min,
                    crossAxisAlignment: CrossAxisAlignment.start,
                    children: [
                      Text(
                        "${session["application_name"] ?? ""} ${session["application_version"] ?? ""}"
                            .trim(),
                        style: const TextStyle(
                          overflow: TextOverflow.fade,
                        ),
                      ),
                      Text(
                        "${session["location"] ?? ""}",
                        style: const TextStyle(
                          overflow: TextOverflow.fade,
                        ),
                      ),
                    ],
                  ),
                );

                if (session["is_current"] == true) {
                  return Column(
                    mainAxisSize: MainAxisSize.min,
                    children: [
                      listTile,
                      ListTile(
                        onTap: () async {
                          var res = await widget
                              .appTelegramTemplateData.telegramClient
                              .invoke(
                            parameters: {
                              "@type": "terminateAllOtherSessions",
                            },
                            isInvokeThrowOnError: false,
                            isVoid: false,
                            telegramClientData: TelegramClientData.tdlib(
                                tdlib_client_id: widget.appTelegramTemplateData
                                    .telegramClient.tdlib.client_id),
                          );
                          res.printPretty();
                          setState(() {});
                        },
                        leading: Icon(Hicons.delete_1_light_outline),
                        title: Text(
                          'Hentikan Semua Sesi Lain',
                          style: const TextStyle(
                            color: Colors.grey,
                          ),
                          textAlign: TextAlign.center,
                        ),
                      ),
                    ],
                  );
                }
                return listTile;
              },
            );

            if (snapshot.hasData) {
              return child;
            }

            return Shimmer.fromColors(
              baseColor: const Color.fromARGB(255, 71, 71, 71),
              highlightColor: context.theme.indicatorColor,
              child: child,
            );
          },
        ),
      ),
    );
  }

  Widget buttonSetting(
      {required IconData iconData,
      required String title,
      String? subtitle,
      void Function()? onPressed}) {
    return MaterialButton(
      onPressed: onPressed,
      child: ListTile(
        leading: Icon(iconData),
        title: Text(title),
        subtitle: (subtitle != null) ? Text(subtitle) : null,
      ),
    );
  }
}
